import ResizeObserver from "resize-observer-polyfill";

const isServer = typeof window === "undefined";

/* istanbul ignore next */
const resizeHandler = function (entries) {
  for (let entry of entries) {
    const listeners = entry.target.__resizeListeners__ || [];
    if (listeners.length) {
      listeners.forEach((fn) => {
        fn();
      });
    }
  }
};

/* istanbul ignore next */
export const addResizeListener = function (element, fn) {
  if (isServer) {
    return;
  }
  if (!element.__resizeListeners__) {
    element.__resizeListeners__ = [];
    element.__ro__ = new ResizeObserver(resizeHandler);
    element.__ro__.observe(element);
  }
  element.__resizeListeners__.push(fn);
};

/* istanbul ignore next */
export const removeResizeListener = function (element, fn) {
  if (!element || !element.__resizeListeners__) {
    return;
  }
  element.__resizeListeners__.splice(
    element.__resizeListeners__.indexOf(fn),
    1
  );
  if (!element.__resizeListeners__.length) {
    element.__ro__.disconnect();
  }
};
/**
 * How to use
 * <el-table height="100px" v-adaptive="{bottomOffset: 30}">...</el-table>
 * el-table height is must be set
 *  bottomOffset: 30(default)   // The height of the table from the bottom of the page.
 */

const doResize = (el, binding, vnode) => {
  const { el: $table } = vnode;

  const { value } = binding;
  if (!value.fixedHeader) {
    return;
  }
  const bottomOffset = (value && value.bottomOffset) || 70;
  if (!$table) {
    return;
  }

  const layout = document.getElementById("my-vite-app");
  const height =
    window.innerHeight - el.getBoundingClientRect().top - bottomOffset;
  const layoutHeight = layout.clientHeight;
  layoutHeight > 710
    ? (layout.style.minHeight = "710px")
    : (layout.style.minHeight = "auto");
  $table.style.height = height + 'px';
  // $table.doLayout();
};

export default {
  beforeMount(el, binding, vnode) {
    el.resizeListener = () => {
      doResize(el, binding, vnode);
    };

    addResizeListener(window.document.body, el.resizeListener);
  },
  mounted(el, binding, vnode) {
    doResize(el, binding, vnode);
  },
  updated(el, binding, vnode) {
    doResize(el, binding, vnode)
  },
  unmounted(el) {
    const layout = document.getElementById("my-vite-app");
    layout.style.minHeight = "710px";
    removeResizeListener(window.document.body, el.resizeListener);
  },
};

