// 得到一个axios实例生成器函数，可配置，不对外暴露，库内部使用

import axios from 'axios' // axios version exactly 0.19.2
import config from '@/config'
import { $ls,$ss } from '@/storage'
const baseUrl = config.baseUrl
/**
 * errorTip 全局错误提示
 */
const errorTipDefault = window.console.error

export default (
  errorTip = errorTipDefault,
  userDefaultConfig = {}
) => {
  // may process the errorTip and userDefaultConfig in later version
  /**
   * 传入两个自定义的拦截器
   */
  return (
    requestHandler,
    responseHander
  ) => {
    const internalConfig = {
      baseURL: baseUrl,
      timeout: 30e3,
      headers: {
        accessToken: $ss.getItem('token')
      }, // 保证在请求拦截里面可能直接访问到headers而不是报错undefined
      // withCredentials: true,
      validateStatus: () => true, // 所有的http返回都收敛
    }
    const mergedCondig = {
      ...internalConfig,
      ...userDefaultConfig,
    }
    const instance = axios.create(mergedCondig)
    // 请求拦截
    instance.interceptors.request.use(
      (config) => {
        if ($ss.getItem("token") != null) {
          config.headers["accessToken"] = $ss.getItem("token");
        }
        return requestHandler(config)
      },
      // 取消此请求
      (error) =>
        new Promise(() => {
          console.error('error in axios.requestInterceptor', error)
        })
    )
    // 响应拦截，收敛全部的请求处理，包括有返回的请求和未返回的请求
    instance.interceptors.response.use(
      (response) => responseHander(response, true), // true 表示一次完整的http请求过程(有来有回)（不管是何种状态码）
      (err) => responseHander(err, false) // 失败的请求（跨域、超时、或其他JS脚本错误导致的请求没法出去）
    )
    // 响应拦截，对用户的响应处理器函数进行错误兜底处理
    instance.interceptors.response.use(
      undefined,
      // 中断
      (error) => {
        console.error('error in axios.responseInterceptor', error)
        errorTip(`api error: ${error?.message || 'unknown error'}`)
        throw error // 重新抛出
      }
    )
    return instance
  }
}
