// 库的暴露出去的方法，需要传入
// 1. 全局错误提示（默认是console.error）
// 2. 请求处理器，在发送http请求时，对axios的config做一些全局或针对性的处理
// 3. 响应处理器，来自服务端的http响应（包括200和非200的全部http status都会到此），以及因为超时被abort取消的Error、因为跨域被取消的Erro以及其他JS代码错误导致请求没法出去的Error

import axiosInstance from './axiosInstance'
import { object2Qs, file2Formdata, self } from './utils'

export default (
  errorTip,
  defaultConfig,
  requestHander = self,
  responseHander = self
) => {
  const axios = axiosInstance(errorTip, defaultConfig)(
    requestHander,
    responseHander
  )
  /**
   *
   * @param type 类型
   * getSimple 简单的get请求，需要一个对象，对象以键值对拼接在URL后面
   * postSimple 简单的post请求，需要一个对象，以www-urlencoded-form方式发送
   * postJson json请求，需要一个对象，以JSON格式发送
   * postFile 上传文件，需要一个formData或File或Blob，如果是File自动封装成名为file值为此File的formData
   * @param url 地址
   * @param data 数据
   * @param opt 其他参数
   */
  const interalRequest = (
    type = 'getSimple',
    url,
    data,
    opt = {}
  ) => {
    const commonConfig = {
      // 不同请求方式可能的公共请求参数
      url,
      responseType: 'json',
    }

    let specialConfig = {}
    switch (type) {
      case 'getSimple':
        specialConfig = {
          method: 'get',
          params: data,
          ...opt,
        }
        break
      case 'postSimple':
        specialConfig = {
          method: 'post',
          data: object2Qs(data),
          ...opt,
        }
        break
      case 'postJson':
        specialConfig = {
          method: 'post',
          data,
          ...opt,
        }
        break
      case 'postFile':
        specialConfig = {
          method: 'post',
          data: file2Formdata(data),
          ...opt,
        }
        break
      case 'custom':
        specialConfig = {
          // do nothing just pass to what received
          ...opt,
        }
        if ((specialConfig.method) === 'get') {
          specialConfig.params = data
        } else {
          specialConfig.data = data
        }
        break
    }

    const finalConfig = {
      ...commonConfig,
      ...specialConfig,
    }

    // @ts-ignore
    return axios(finalConfig)
  }

  const getSimple = (url, data = {}, opt = {}) => {
    return interalRequest('getSimple', url, data, opt)
  }

  const postSimple = (url, data = {}, opt = {}) => {
    return interalRequest('postSimple', url, data, opt)
  }

  const postJson = (url, data = {}, opt = {}) => {
    return interalRequest('postJson', url, data, opt)
  }

  const postFile = (url, data = new window.Blob([]), opt = {}) => {
    return interalRequest('postFile', url, data, opt)
  }

  const request = (method = 'get', url, data, opt = {}) => {
    return interalRequest('custom', url, data, {
      method: method.toLowerCase(),
      ...opt,
    })
  }

  return {
    getSimple,
    postSimple,
    postJson,
    postFile,
    request
  }
}
