import dayjs from 'dayjs'
import { formateData } from './utils'

import localStorageEngine from './localStorage'
import sessionStorageEngine from './sessionStorage'

const engineMap = {
  local: localStorageEngine,
  session: sessionStorageEngine,
}

const self = (v) => v

/******
 *
 * @param type 存储类型，local = localStorage session = sessionStorage
 * @param prefix 命名空间 配置 存储名称前缀
 * @param encrypt 是否支持加密，传入一个对象，方法en用于加密，方法de用于解密
 *
 * ******/

export default (
  type,
  prefix = '',
  encrypt
) => {
  const engine = engineMap[type]
  encrypt = encrypt ?? { en: self, de: self }
  return {
    engine,
    prefix,
    encrypt,
    /**
     * @param name
     * @param value
     * @param expire 过期时间，单位毫秒
     * @param encrypt 是否要加密
     */
    setItem(name, value, expire = -1, encrypt = false) {
      name = `${this.prefix}__${name}`
      const data = formateData(value, expire)
      if (encrypt) {
        data.data = this.encrypt.en(data.data)
        data.encrypt = 1
      }
      this.engine.setItem(name, JSON.stringify(data))
    },
    getItem(name) {
      name = `${this.prefix}__${name}`
      let dataRead = this.engine.getItem(name)
      if (dataRead === null) return null // 不存在
      const data = JSON.parse(dataRead)
      const now = dayjs().valueOf()
      if (data.expire < now && data.expire !== -1) {
        // 过期了
        this.engine.removeItem(name) // 删除
        return null
      }
      if (data.encrypt) {
        // 存在加密
        data.data = this.encrypt.de(data.data)
        data.encrypt = 0
      }
      let result = null
      try {
        result = JSON.parse(data.data)
      } catch (error) {
        result = data.data
      }
      return result
    },
    removeItem(name) {
      name = `${this.prefix}__${name}`
      this.removeItemNative(name)
    },
    // 原始的setItem
    setItemNative: engine.setItem.bind(engine),
    // 原始的getItem
    getItemNative: engine.getItem.bind(engine),
    // 原始的removeItem
    removeItemNative: engine.removeItem.bind(engine),
  }
}
