import getRequest from '@/plugins/request'
import config from '@/config'
import { wipeNulish } from '@/utils'
import { ElMessage } from 'element-plus'
import { $ls } from "@/storage";

// 全局的错误提示，默认是console.error
const errorTip = (msg) => {
  ElMessage.error(msg)
}

const requestHandler = (config) => {
  // 检查网络状态
  if (!window.navigator.onLine) {
    throw Error('没有网络链接！请检查网络连接！然后刷新重试')
  }
  // 如果参数有null和undefined就删除
  if (config.method === 'get') {
    config.params = wipeNulish(config.params)
  } else {
   if (config.method !== 'put') {
        const { data } = config
        const shouldWipeNulish = !(
        data instanceof File ||
        data instanceof Blob ||
        data instanceof FormData ||
        data instanceof URLSearchParams
      )
       config.data = shouldWipeNulish ? wipeNulish(config.data) : data
   }
    
  }
  return config
}

const responseHandler = (response, type) => {
  // // 如果是导出就把头部字段加进去
  if (response.data instanceof Blob) {
     response.data = {
      data: response.data,
      headers: response.headers
     }
  }
  // type = true response的后端返回的结果，不管这个response的状态码是何种
  // type = false 表示请求出错了，可能是因为超时，也可能是跨域，也可能是脚本错误，但是两面的2个在开发和测试时候已经避免掉了
  // 200 ok { code: number, data: Object, message: string }
  // 201 created
  // 401 unauthorized
  // 403 forbidden
  // 404 notfound

  // 没有收到
  if (!type) {
    // 在生产环境只有超时错误，其他错误在开发和测试就被避免掉了
    const err = response
    throw Error('网络异常：' + err.message)
  }
  // 有response
  if (type) {
    // 响应错误
    const httpStatus = +response?.status
    switch (httpStatus) {
      case 200:
        return response.data
      case 201:
        return response
      case 401:
        throw Error('没有足够的权限')
      case 403:
        throw Error('禁止访问')
      case 404:
        throw Error('未找到有效资源')
      case 424:
          window.location.href = '/login'
          throw Error('token已失效,请重新登录');
      case 500:
        throw Error('服务器暂时无法连接，请稍后再试')
    }
    if (!isNaN(httpStatus) && (httpStatus < 200 || httpStatus > 299)) {
      throw Error('未知异常，请刷新重试')
    }
  }
  // 兜底，让它成功，返回给外部的处理函数
  return response
}

const { getSimple, postSimple, postJson, postFile, request } = getRequest(
  errorTip,
  {
    timeout: 30e3,
    baseURL: config.baseUrl,
  },
  requestHandler,
  responseHandler
)

export { getSimple, postSimple, postJson, postFile, request }
