import _ from 'lodash'
import { openLoading, closeLoading } from '@/utils/loading.js'
import { getSimple, postSimple, request } from "@/request";


/**
 * 抹除api请求的对象的null和undefined
 */
export const wipeNulish = (obj) => {
  obj = Object(obj) // 不是对象类型就转成对象类型
  obj = _.cloneDeep(obj) // 不改变原来的值
  _(obj).keys().forEach((key) => {
    // 删除null和undefined
    obj[key] == null && delete obj[key]
    if (
      typeof obj[key] === 'object' &&
      Object.prototype.toString.call(obj) === '[object Object]'
    ) {
      // 不对Array或Date或RegExp这样的对象处理
      // 递归删除
      wipeNulish(obj[key])
    }
  })
  return obj
}

/**
 * 格式化globEager返回的文件名,'./xx/yy.xx' -> 'yy'
 * 再把导入的模块的默认导出(export default)给对应的键值对
 */
export const filterGlobEagerFilename = (obj) => {
  return _(obj)
    .mapKeys((_, i) => {
      const arr = i.split('/')
      return arr[arr.length - 1]
    })
    .mapKeys((_, i) => i.match(/(.+)\.\w+$/)[1])
    .mapValues((i) => i.default)
    .value()
}

// 获取assets静态资源
export const getAssetsFile = (url) => {
  return new URL(`../assets/img/${url}`,
    import.meta.url).href
}

// 精确计算加法
export function acc(arg1, arg2) {
  let r1 = deal(arg1)
  let r2 = deal(arg2)
  let m = Math.pow(10, Math.max(r1, r2))
  let res1 = numMulti(arg1, m)
  let res2 = numMulti(arg2, m)
  return (res1 + res2) / m
}

export function deal(arg) {
  var t = 0
  try {
    t = arg.toString().split(".")[1].length
  } catch (e) { }
  return t
}

// 乘法精度计算
export function numMulti(num1, num2) {
  var baseNum = 0
  try {
    baseNum += num1.toString().split(".")[1].length
  } catch (e) { }
  try {
    baseNum += num2.toString().split(".")[1].length
  } catch (e) { }
  return Number(num1.toString().replace(".", "")) * Number(num2.toString().replace(".", "")) / Math.pow(10, baseNum)
}

export const routerPath = (menuList) => {
  let arrList = [];
  function childrenServser(menuList) {
    menuList && menuList.forEach((item) => {
      if (item.children) return childrenServser(item.children)
      else {
        arrList.push(item.path)
      }
    })
  }
  childrenServser(menuList)
  return arrList
}

// 导出
export const downloadFn = (allParams) => {
  const {
    url,
    params,
  } = allParams;
  let opt = {
    responseType: 'blob',
  }
  openLoading()
  return new Promise((resolve, reject) => {
    getSimple(
      url,
      params,
      opt
    ).then(res => {
      let fileName = decodeURI(res.headers["content-disposition"].split(";")[1].split("filename=")[1])
      const link = document.createElement('a');
      let blob = new Blob([res.data], { type: 'application/vnd.ms-excel' });
      link.style.display = 'none';
      link.href = URL.createObjectURL(blob);
      link.setAttribute('download', fileName);
      link.download = fileName || `下载文件.xls`;
      document.body.appendChild(link);
      link.click();
      document.body.removeChild(link)
      closeLoading()
      ElMessage({
        message: '导出成功',
        type: 'success',
        duration: 3000,
      });
      resolve(res);
    }).catch((errorMsg) => {
      closeLoading()
      ElMessage({
        message: errorMsg,
        type: 'error',
        duration: 3000,
      });
      reject(errorMsg);
    })
  })
}
export const accDivCoupon = (arg1, arg2) => {
  var t1 = 0, t2 = 0, r1, r2;
  try { t1 = arg1.toString().split(".")[1].length } catch (e) { }
  try { t2 = arg2.toString().split(".")[1].length } catch (e) { }
  r1 = Number(arg1.toString().replace(".", ""));
  r2 = Number(arg2.toString().replace(".", ""));
  return (r1 / r2) * Math.pow(10, t2 - t1);
}
/**
 * 保留两位小数点后两位
 * <!--在双括号中这样使用即可-->
<div class="col">{{item.dataToday | numFilter}}</div>
 */
export const numFilter = (value) => {
  let realVal = "";
  if (!isNaN(value) && value !== "") {
    // 截取当前数据到小数点后两位,改变toFixed的值即可截取你想要的数值
    realVal = parseFloat(value).toFixed(2);
  } else {
    realVal = "--";
  }
  return realVal;
}

// 重置 key
export const reset = (obj) => {
  for (var key in obj) {
    delete obj[key];
  }
}

/**
 * @description 函数节流
 * @param {Function} func 函数
 * @param {Number} wait 延迟执行毫秒数,默认200
 */
export const VueThrottle = (func, wait = 200) => {
  let timeout;
  return function () {
    let that = this;
    let args = arguments;
    if (!timeout) {
      timeout = setTimeout(() => {
        timeout = null;
        func.apply(that, args)
        ElMessage.closeAll();// 关闭所有弹窗
      }, wait)
    }
    else {
      ElMessage.warning("操作太频繁了哦，稍后再试");
    }
  }
}